#include "mainwindow.h"
#include "ui_mainwindow.h"


// ____________________
// ::: Class Button :::

/*!
    \brief Constructor : load images and set initial status
*/
Button::Button()
{
    Status=false;
    this->img_Button_On.load(":/images/button_on.png");
    this->img_Button_Off.load(":/images/button_off.png");
    this->setPixmap(this->img_Button_Off);
}

/*!
    \brief  When the button is pressed, the status and the display are changed
*/
void Button::mousePressEvent(QMouseEvent * e)
{
  QLabel::mousePressEvent(e);  
  Status=!Status;
  if (Status)
      this->setPixmap(this->img_Button_On);
  else
      this->setPixmap(this->img_Button_Off);
  emit clicked(e->pos());
}



// _________________
// ::: Class led :::

/*!
    \brief Constructor : load images and set initial status
*/
Led::Led()
{
    Status=false;
    this->img_Led_On.load(":/images/led_on.png");
    this->img_Led_Off.load(":/images/led_off.png");
    this->setPixmap(this->img_Led_Off);
}


/*!
    \brief  Set the current led on or off
*/
void Led::setStatus(bool bStatus)
{
    this->Status=bStatus;
    if (Status)
        this->setPixmap(this->img_Led_On);
    else
        this->setPixmap(this->img_Led_Off);
}



// ________________________
// ::: Class MainWindow :::

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    // Fix the window size
    this->setFixedSize(390,170);
    this->setWindowTitle("Boitier I/O Button/Led");

    // Add buttons in the window
    for (int i=0;i<8;i++)
        this->ui->LayoutButtons->addWidget(&tButtons[7-i]);

    // Add leds in the window
    for (int i=0;i<8;i++)
        this->ui->LayoutLeds->addWidget(&tLeds[7-i]);

    // Create and initialize udp socket
    udpSocket = new QUdpSocket(this);
    if (!udpSocket->bind(UDP_PORT, QUdpSocket::DontShareAddress))
    {
        std::cerr << "Error while binding socket (already launch ?)\n";
        exit(-1);
    }

    // Connect the socket to processpendingDatagrams when data is received
    connect(udpSocket, SIGNAL(readyRead()),this, SLOT(processPendingDatagrams()));
}




/*!
    \brief  Read pending datagrams
*/
void MainWindow::processPendingDatagrams()
{
    // For each pending datagram
    while (udpSocket->hasPendingDatagrams())
    {
        // Read data
        char Data[3];
        udpSocket->readDatagram(Data, 2);

        // Write led status
        if (Data[0]=='L')
        {
            for (int i=0;i<8;i++)
                tLeds[i].setStatus( Data[1] & (1<<i) );           
        }

        // Read led status
        if (Data[0]=='B')
        {
            // Read the buttons
            unsigned char ButStatus=0;
            for (int i=0;i<8;i++)
                ButStatus=ButStatus | (tButtons[i].getStatus()<<i);

            // Send the status to the client
            char Data[3];
            QUdpSocket udpSocket;
            sprintf (Data,"B%c",ButStatus);
            udpSocket.writeDatagram(Data, strlen(Data)+1,QHostAddress::LocalHost, UDP_PORT+1);           
        }
    }
}




MainWindow::~MainWindow()
{
    delete ui;
    delete (udpSocket);
}
