#include "driver_led_inter.h"


// Socket for communications
QUdpSocket      udpSocketRead;





/*!
    \brief  Read data from the button
*/
bool readButton(unsigned char* Val)
{
    // Prepare the message
    QUdpSocket udpSocket;
    char Data[3];
    sprintf (Data,"B0");

    // Send the request
    udpSocketRead.flush();
    udpSocket.writeDatagram(Data, strlen(Data)+1,QHostAddress::LocalHost, UDP_PORT);

    // Wait 100 ms
    QTime dt;
    dt.start();
    while (dt.elapsed()<TIME_OUT_MS)
    {
        // If a data is received
        if (udpSocketRead.hasPendingDatagrams())
        {
            // Read data and return true
            char Data[3];
            udpSocketRead.readDatagram(Data, 2);
            *Val=Data[1];
            return true;
        }
    }
    // No data received, return false
    return false;
}





/*!
    \brief      Open communication with the led/button interface
    \return     1 success
                -1 error while binding socket
                -2 error during communication
 */
unsigned char       OpenPortInter    ()
{
    // Bind the socket
    if (!udpSocketRead.bind(UDP_PORT+1, QUdpSocket::DontShareAddress)) return -1;

    // Check if the connection is OK
    unsigned char Dumb;
    if (!readButton(&Dumb)) return -2;
    return 1;
}




/*!
    \brief      Send a value (to the led/button interface) : change led status)
    \param      val is the led status
*/
void              OutPortInter  (unsigned char val)
{
    char Data[3];
    QUdpSocket udpSocket;
    sprintf (Data,"L%c",val);
    udpSocket.writeDatagram(Data, strlen(Data)+1,QHostAddress::LocalHost, UDP_PORT);
    usleep(2);
}



/*!
    \brief      Request a value (from the led/button interface) : get buttons status
    \return     The button status
*/
unsigned char     InPortInter   ()
{
    unsigned char Val=0;
    readButton(&Val);
    return Val;
}



/*!
    \brief      Close the connection with the led/button interface
*/
void              ClosePortInter   ()
{
    udpSocketRead.close();
}
